"""
  Copyright (c) 2022 LG Electronics Inc.
  SPDX-License-Identifier: MIT
"""

"""
webos-emulator: webOS Emulator Launcher
"""

__author__ = """Heegoo Han"""
__email__ = 'heegoo.han@lge.com'
__version__ = '0.1.0'  # TODO: is this needed?

from webos_emulator.version import __version__
from webos_emulator.__main__ import WebosEmulator
